ALTER FUNCTION IF EXISTS FUNCTION1(NUMBER) RENAME TO FUNCTION2;
ALTER FUNCTION FUNCTION2(NUMBER) SET SECURE;
ALTER FUNCTION FUNCTION3(NUMBER) RENAME TO FUNCTION3B;
ALTER FUNCTION FUNCTION4(NUMBER) SET API_INTEGRATION = API_INTEGRATION_2;
ALTER FUNCTION FUNCTION5(NUMBER) SET MAX_BATCH_ROWS = 100;
ALTER FUNCTION FUNCTION6(NUMBER) SET COMPRESSION = GZIP;
ALTER FUNCTION FUNCTION7(NUMBER) SET REQUEST_TRANSLATOR = TRANSLATOR_FUNCTION;
ALTER FUNCTION FUNCTION8(NUMBER) SET RESPONSE_TRANSLATOR = TRANSLATOR_FUNCTION;
ALTER FUNCTION FUNCTION9(NUMBER) SET HEADERS = ('abc' = 'def');
ALTER FUNCTION FUNCTION10(NUMBER) SET CONTEXT_HEADERS = (CURRENT_ROLE,CURRENT_TIMESTAMP);
ALTER FUNCTION FUNCTION11(NUMBER) SET COMMENT = 'Woohoo!';
