-- Testing of the specific create table end options
CREATE MULTISET TABLE NUM_LTR_DESVINCULADOS_ADH
(
    DES_EVENTO VARCHAR(255) CHARACTER SET LATIN NOT CASESPECIFIC COMPRESS ('Cambio de bandera', 'Cierre'),
    IND_CONTINUA BYTEINT COMPRESS
 )
PRIMARY INDEX( COD_TARJETA, COD_EST, FEC_CIERRE_EST, IND_TIPO_TARJETA )
PARTITION BY RANGE_N (FEC_OPERACION BETWEEN DATE '2007-01-01' AND DATE '2022-01-01' EACH INTERVAL '1' MONTH, NO RANGE OR UNKNOWN)
INDEX HOPR_TRN_TRAV_SIN_MP_I ( IND_TIPO_TARJETA );