CACHE TABLE TESTCACHE
OPTIONS ('storageLevel' 'DISK_ONLY')
SELECT
    A,
    B
FROM TESTDATA;

CACHE LAZY TABLE TESTCACHE
OPTIONS ('storageLevel' 'DISK_ONLY')
SELECT A FROM TESTDATA;

CACHE TABLE TESTCACHE
OPTIONS ('storageLevel' 'DISK_ONLY') AS
SELECT A FROM TESTDATA;

CACHE TABLE TESTCACHE
OPTIONS ('storageLevel' = 'DISK_ONLY') AS
SELECT A FROM TESTDATA;
