PURGE BINARY LOGS TO 'mysql-bin.010';

PURGE BINARY LOGS BEFORE '2019-04-02 22:46:26';
PURGE BINARY LOGS BEFORE DATETIME '2019-04-02 22:46:26';
PURGE BINARY LOGS BEFORE TIMESTAMP '2019-04-02 22:46:26';

PURGE BINARY LOGS BEFORE 19830905132800;
PURGE BINARY LOGS BEFORE DATETIME 19830905132800;
PURGE BINARY LOGS BEFORE TIMESTAMP 19830905132800;
