CHECK TABLE some_table FOR UPGRADE;

CHECK TABLE some_table1, some_table2 FOR UPGRADE;

CHECK TABLE some_table QUICK;

CHECK TABLE some_table1, some_table2 QUICK;

CHECK TABLE some_table FAST;

CHECK TABLE some_table1, some_table2 FAST;

CHECK TABLE some_table MEDIUM;

CHECK TABLE some_table1, some_table2 MEDIUM;

CHECK TABLE some_table EXTENDED;

CHECK TABLE some_table1, some_table2 EXTENDED;

CHECK TABLE some_table CHANGED;

CHECK TABLE some_table1, some_table2 CHANGED;

CHECK TABLE some_table FAST QUICK;
