CREATE TABLE DEST.MELDER(
      [ID] INT PRIMARY KEY NOT NULL DEFAULT NEXT VALUE FOR [dbo].[SEQ_MELDER]
      ,[DOWNLOADED_TOTAL] INT NULL
      ,[WARNED_DAILY] INT NULL
      ,[DATE_OF_REPORT] DATETIME NULL
      ,DATE_LAST_INSERTED DATETIME DEFAULT GETDATE()
);
