DESC DATASHARE salesshare;

DESC DATASHARE salesshare of ACCOUNT '123456789012' NAMESPACE '13b8833d-17c6-4f16-8fe4-1a018f5ed00d';

DESC DATASHARE salesshare of NAMESPACE '13b8833d-17c6-4f16-8fe4-1a018f5ed00d';
