-- Testing of the specific column options
CREATE MULTISET TABLE TABLE_2
(
    CHAR_FIELD CHAR(19) CHARACTER SET LATIN NOT CASESPECIFIC NOT NULL,
    DATE_FIELD DATE FORMAT 'YYYY-MM-DD' NOT NULL,
    BYTE_FIELD BYTEINT COMPRESS 0,
    DECIMAL_FIELD DECIMAL(15, 2) COMPRESS (50.00, 45.50, 40.00, 30.00, 27.80, 27.05, 20.00, 17.87, 17.56, 17.41, 17.26, 17.11, 16.96, 16.82, 16.68),
    TIMESTAMP_FIELD TIMESTAMP(6) NOT NULL
)
PRIMARY INDEX( CHAR_FIELD, DATE_FIELD, BYTE_FIELD );