-- Lists all the databases.
SHOW DATABASES;

-- Lists databases with name starting with string pattern `pay`
SHOW DATABASES LIKE 'pay*';

-- Lists all databases. Keywords SCHEMAS and DATABASES are interchangeable.
SHOW SCHEMAS;
