SELECT DATE_ADD(CURDATE(), INTERVAL -30 DAY);

SELECT SUBDATE('2008-01-02', INTERVAL 31 DAY);

SELECT ADDDATE(CURDATE(), INTERVAL -30 DAY);

SELECT DATE_SUB('1992-12-31 23:59:59.000002', INTERVAL '1.999999' SECOND_MICROSECOND);

SELECT DATE_ADD('2100-12-31 23:59:59', INTERVAL '1:1' MINUTE_SECOND);

SELECT DATE_ADD(CURDATE(), INTERVAL 7 * 4 DAY);

SELECT
    ADDDATE(CURDATE(), INTERVAL col1 DAY)
FROM
    tbl1
;

SELECT
    SUBDATE(CURDATE(), INTERVAL col1 + col2 DAY)
FROM
    tbl1
;
