COMMENT ON COLUMN employees.job_id
   IS 'abbreviated job title';
COMMENT ON TABLE employees
   IS 'employees table';
COMMENT ON INDEXTYPE employees_indextype
   IS 'employees indextype';
COMMENT ON OPERATOR employees_operator
   IS 'employees operator';
COMMENT ON MATERIALIZED VIEW employees_mv
   IS 'employees materialized view';
