ANALYZE TABLE students COMPUTE STATISTICS NOSCAN;

ANALYZE TABLE students COMPUTE STATISTICS;

ANALYZE TABLE students PARTITION (student_id = 111111) COMPUTE STATISTICS;

ANALYZE TABLE students COMPUTE STATISTICS FOR COLUMNS name;

ANALYZE TABLES IN school_db COMPUTE STATISTICS NOSCAN;

ANALYZE TABLES COMPUTE STATISTICS;
