CREATE TABLE t (c1 DECIMAL);
CREATE TABLE t (c1 DECIMAL(10));
CREATE TABLE t (c1 DECIMAL(10,2));
CREATE TABLE t (c1 DEC(10,2));
CREATE TABLE t (c1 NUMERIC(10));
CREATE TABLE t (c1 NUMBER(10,2));
CREATE TABLE t (c1 BIGINT);
CREATE TABLE t (c1 DOUBLE);
CREATE TABLE t (c1 DOUBLE PRECISION);
CREATE TABLE t (c1 FLOAT);
CREATE TABLE t (c1 INT);
CREATE TABLE t (c1 INTEGER);
CREATE TABLE t (c1 REAL);
CREATE TABLE t (c1 SHORTINT);
CREATE TABLE t (c1 TINYINT);
CREATE TABLE t (c1 SMALLINT);
CREATE TABLE t (c1 BOOL);
CREATE TABLE t (c1 BOOLEAN);
CREATE TABLE t (c1 DATE);
CREATE TABLE t (c1 TIMESTAMP);
CREATE TABLE t (c1 TIMESTAMP WITH LOCAL TIME ZONE);
CREATE TABLE t (c1 INTERVAL YEAR (1) TO MONTH);
CREATE TABLE t (c1 INTERVAL DAY (2) TO SECOND(1));
CREATE TABLE t (c1 GEOMETRY(1000));
CREATE TABLE t (c1 HASHTYPE);
CREATE TABLE t (c1 HASHTYPE(8 BYTE));
CREATE TABLE t (c1 HASHTYPE(8 BIT));
CREATE TABLE t (c1 CHAR(1));
CREATE TABLE t (c1 CHAR VARYING (1));
CREATE TABLE t (c1 VARCHAR(2000 CHAR));
CREATE TABLE t (c1 VARCHAR2(2000));
CREATE TABLE t (c1 VARCHAR(2000 BYTE));
CREATE TABLE t (c1 LONG VARCHAR);
CREATE TABLE t (c1 CHARACTER LARGE OBJECT(1000));
CREATE TABLE t (c1 CHARACTER VARYING(1000));
CREATE TABLE t (c1 CLOB(2000));
CREATE TABLE t (c1 CLOB(2000) ASCII);
CREATE TABLE t (c1 VARCHAR(2000 CHAR) UTF8);
