PRELOAD TABLE t(i);
PRELOAD DATABASE;
PRELOAD TABLES t1,t2;
PRELOAD SCHEMAS s1,s2;
PRELOAD SCHEMA s1;
TRUNCATE AUDIT LOGS;
TRUNCATE AUDIT LOGS KEEP LAST MONTH;
TRUNCATE AUDIT LOGS KEEP FROM '2019-01-01';
