DROP CONNECTION MY_SCHEMA.T1;

DROP CONNECTION IF EXISTS MY_SCHEMA.T1;

DROP DATABASE MYTESTDB2;

DROP DATABASE IF EXISTS MYTESTDB2;

DROP DATABASE IF EXISTS MYTESTDB2 CASCADE;

DROP EXTERNAL TABLE MY_SCHEMA.T1;

DROP EXTERNAL TABLE IF EXISTS MY_SCHEMA.T1;

DROP EXTERNAL TABLE IF EXISTS MY_SCHEMA.T1 RESTRICT;

DROP FILE FORMAT MY_SCHEMA.MY_FORMAT;

DROP FILE FORMAT IF EXISTS MY_SCHEMA.MY_FORMAT;

DROP FUNCTION MY_SCHEMA.MY_FUNCTION(NUMBER, NUMBER);

DROP FUNCTION IF EXISTS MY_SCHEMA.MY_FUNCTION(NUMBER, NUMBER);

DROP INTEGRATION T2;

DROP API INTEGRATION IF EXISTS T2;

DROP MANAGED ACCOUNT READER_ACCT1;

DROP MASKING POLICY SSN_MASK;

DROP MATERIALIZED VIEW MY_SCHEMA.MV1;

DROP MATERIALIZED VIEW IF EXISTS MY_SCHEMA.MV1;

DROP NETWORK POLICY MY_POLICY;

DROP NETWORK POLICY IF EXISTS MY_POLICY;

DROP PIPE MY_SCHEMA.MYPIPE;

DROP PIPE IF EXISTS MY_SCHEMA.MYPIPE;

DROP PROCEDURE MY_SCHEMA.ADD_ACCOUNTING_USER(VARCHAR);

DROP PROCEDURE IF EXISTS MY_SCHEMA.ADD_ACCOUNTING_USER(VARCHAR);

DROP RESOURCE MONITOR MY_MONITOR_RESOURCE;

DROP ROLE MYROLE;

DROP ROLE IF EXISTS MYROLE;

DROP ROW ACCESS POLICY RAP_TABLE_EMPLOYEE_INFO;

DROP ROW ACCESS POLICY IF EXISTS RAP_TABLE_EMPLOYEE_INFO;

DROP SCHEMA MY_SCHEMA;

DROP SCHEMA IF EXISTS MY_SCHEMA;

DROP SCHEMA IF EXISTS MY_SCHEMA CASCADE;

DROP SEQUENCE MY_SCHEMA.INVOICE_SEQUENCE_NUMBER;

DROP SEQUENCE IF EXISTS MY_SCHEMA.INVOICE_SEQUENCE_NUMBER;

DROP SEQUENCE IF EXISTS MY_SCHEMA.INVOICE_SEQUENCE_NUMBER CASCADE;

DROP SESSION POLICY SESSION_POLICY_PRODUCTION_1;

DROP SESSION POLICY IF EXISTS SESSION_POLICY_PRODUCTION_1;

DROP SHARE SALES_S;

DROP STAGE MY_SCHEMA.MY_STAGE;

DROP STAGE IF EXISTS MY_SCHEMA.MY_STAGE;

DROP STREAM MY_SCHEMA.T2;

DROP STREAM IF EXISTS MY_SCHEMA.T2;

DROP TABLE MY_SCHEMA.T2;

DROP TABLE IF EXISTS MY_SCHEMA.T2;

DROP TABLE IF EXISTS MY_SCHEMA.T2 CASCADE;

DROP TAG COST_CENTER;

DROP TAG IF EXISTS COST_CENTER;

DROP TASK MY_SCHEAM.T2;

DROP TASK IF EXISTS MY_SCHEAM.T2;

DROP USER USER1;

DROP USER IF EXISTS USER1;

DROP VIEW MY_SCHEMA.MY_VIEW;

DROP VIEW IF EXISTS MY_SCHEMA.MY_VIEW;

DROP WAREHOUSE MY_WAREHOUSE;

DROP WAREHOUSE IF EXISTS MY_WAREHOUSE;
