SELECT *
FROM table1
NATURAL JOIN table2;

SELECT *
FROM table1
NATURAL INNER JOIN table2;

SELECT *
FROM table1
NATURAL LEFT JOIN table2;

SELECT *
FROM table1
NATURAL LEFT OUTER JOIN table2;

SELECT *
FROM table1
NATURAL RIGHT JOIN table2;

SELECT *
FROM table1
NATURAL RIGHT OUTER JOIN table2;

SELECT *
FROM table1
NATURAL FULL JOIN table2;

SELECT *
FROM table1
NATURAL FULL OUTER JOIN table2;
