select my_field from my_table_asdlfkhsldfjkhsadlfjksahdf /* this is a long multi-line comment that seeks dajkfhsalkjfdhs
    to explain why we are selecting from this table sfafdjkafdksajdfhsdajkfhsakldjfhsakldjfhksdajf
rather than another table
    dsafdafsdfsadfsadfasdfasfdsadfasdfa fdsaf sdafa sadf asdf sdfa sdf asdf */ my_table
