CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b';
CREATE USER admin WITH PASSWORD 'sha256|Mypassword1';
CREATE USER lazy PASSWORD DISABLE;
CREATE USER lazy WITH PASSWORD DISABLE;

CREATE USER "lazy" WITH PASSWORD DISABLE;

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' CREATEDB;
CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' NOCREATEDB;

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' CREATEUSER;
CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' NOCREATEUSER;

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' SYSLOG ACCESS RESTRICTED;
CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' SYSLOG ACCESS UNRESTRICTED;

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' IN GROUP group_1;
CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' IN GROUP group_1, group_2;

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' VALID UNTIL '2017-06-10';

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' CONNECTION LIMIT 30;
CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' CONNECTION LIMIT UNLIMITED;

CREATE USER user1 PASSWORD 'md5153c434b4b77c89e6b94f12c5393af5b' SESSION TIMEOUT 120;
