ALTER DEFAULT PRIVILEGES FOR USER the_user IN SCHEMA the_schema GRANT EXECUTE ON PROCEDURES TO the_other_user;
ALTER DEFAULT PRIVILEGES FOR USER the_user IN SCHEMA the_schema GRANT EXECUTE ON ROUTINES TO the_other_user;
