
COPY (SELECT * FROM t1 WHERE value < 100) TO STDOUT;

COPY (SELECT * FROM t1 WHERE value < 100) TO STDOUT WITH (FORMAT binary);

COPY (SUBSCRIBE some_view) TO STDOUT;

COPY (SUBSCRIBE some_view) TO STDOUT WITH (FORMAT binary);

COPY (VALUES (1, '2'), (3, '4'), (5, '\\\t\n\rtest\\N'), (6, NULL) ORDER BY column1) TO STDOUT;

COPY t FROM STDIN;

COPY t FROM STDIN WITH (FORMAT CSV, DELIMITER '!', QUOTE '!');

COPY t FROM STDIN WITH (DELIMITER '|');

COPY t FROM STDIN (FORMAT CSV);

COPY t FROM STDIN (DELIMITER '|');
