/* birds /* live /* in /* nests */ */  */fdsfdlsjf */

/* although
/* so
/**/ do */
wasps
*/

SELECT foo
/*nest/*/*nest/*/*/*
nest nest /* */*/*/*/nest*/*/*/ FROM bar

-- A recursive block comment (fun patternwise - no actual recursion going on ofc)
/* A block comment looks like /* A block comment looks like /* A block comment looks like /* A block comment looks like /* A block comment looks like /* ... */ */ */ */ */ */

-- Test cases from #2086.
/**
**/

/**
(
**/

/**
'
**/
