ALTER VIRTUAL SCHEMA s2
    SET CONNECTION_STRING = 'jdbc:hive2://localhost:10000/default';
ALTER VIRTUAL SCHEMA s2 REFRESH;
ALTER VIRTUAL SCHEMA s1 CHANGE OWNER "role1";
ALTER VIRTUAL SCHEMA s1 CHANGE OWNER role1;
