drop table some_table;

drop table if exists some_table;

drop table some_table restrict;

drop table if exists some_table restrict;

drop table some_table cascade;

drop table if exists some_table cascade;
