ALTER TABLE t ADD COLUMN IF NOT EXISTS new_dec DECIMAL(18,0);
ALTER TABLE t ADD (new_char CHAR(10) DEFAULT 'some text');
ALTER TABLE myschema.t DROP COLUMN i;
ALTER TABLE t DROP j;
ALTER TABLE t MODIFY (i DECIMAL(10,2));
ALTER TABLE t MODIFY (j VARCHAR(5) DEFAULT 'text');
ALTER TABLE t MODIFY k INTEGER IDENTITY(1000);
ALTER TABLE t RENAME COLUMN i TO j;
ALTER TABLE t ALTER COLUMN v SET DEFAULT CURRENT_USER;
ALTER TABLE "SCHEMA"."TABLE" ALTER COLUMN v DROP DEFAULT;
ALTER TABLE t ALTER COLUMN id SET IDENTITY 1000;
ALTER TABLE t ALTER COLUMN id DROP IDENTITY;
