ALTER SEQUENCE IF EXISTS foo AS integer;

ALTER SEQUENCE foo INCREMENT BY 4 NO MINVALUE MAXVALUE 56;

ALTER SEQUENCE foo INCREMENT 3 NO MAXVALUE MINVALUE 3;

ALTER SEQUENCE foo START 7 CACHE 4;

ALTER SEQUENCE foo RESTART WITH 14 NO CYCLE;

ALTER SEQUENCE foo OWNED BY foo.foo;

ALTER SEQUENCE foo OWNED BY NONE;

ALTER SEQUENCE IF EXISTS foo OWNER TO my_user;

ALTER SEQUENCE foo OWNER TO CURRENT_USER;

ALTER SEQUENCE foo OWNER TO SESSION_USER;

ALTER SEQUENCE foo RENAME TO foo2;

ALTER SEQUENCE foo SET SCHEMA my_schema;
