ALTER SESSION SET TIME_ZONE='EUROPE/BERLIN';
ALTER SESSION SET QUERY_TIMEOUT=120;
ALTER SESSION SET NLS_DATE_FORMAT='DDD-YYYY';
ALTER SESSION SET SESSION_TEMP_DB_RAM_LIMIT = '10240M';
ALTER SESSION SET SNAPSHOT_MODE = 'OFF';
ALTER SESSION SET SNAPSHOT_MODE = 'SYSTEM TABLES';
ALTER SESSION SET IDLE_TIMEOUT = 2400;
