CREATE VIEW v1 (c,d) AS 
SELECT a,b FROM t1;

CREATE OR REPLACE VIEW v1 (c,d,e,f) AS 
SELECT a,b, a IN (SELECT a+2 FROM t1), a = all (SELECT a FROM t1) FROM t1;

CREATE VIEW v2 AS SELECT a FROM t1 WITH CASCADED CHECK OPTION;
