ALTER SYSTEM SET NLS_DATE_LANGUAGE='DEU';
ALTER SYSTEM SET NLS_FIRST_DAY_OF_WEEK=1;
ALTER SYSTEM SET SNAPSHOT_MODE = 'SYSTEM TABLES';
ALTER SYSTEM SET IDLE_TIMEOUT = 3600;
ALTER SYSTEM SET USER_TEMP_DB_RAM_LIMIT = '50G';
