DROP INDEX `test` ON `table1`.`foo` ALGORITHM = DEFAULT;
DROP INDEX `test` ON `table1`.`foo` ALGORITHM = INPLACE;
DROP INDEX `test` ON `table1`.`foo` ALGORITHM = COPY;
DROP INDEX `test` ON `table1`.`foo` ALGORITHM DEFAULT;
DROP INDEX `test` ON `table1`.`foo` ALGORITHM INPLACE;
DROP INDEX `test` ON `table1`.`foo` ALGORITHM COPY;
