REINDEX INDEX my_index;

REINDEX TABLE my_table;

REINDEX DATABASE broken_db;

REINDEX TABLE CONCURRENTLY my_broken_table;

REINDEX (CONCURRENTLY FALSE) SYSTEM mydb;

REINDEX (TABLESPACE my_tablespace, VERBOSE TRUE) SCHEMA my_schema;
