INSERT OVERWRITE LOCAL DIRECTORY '/tmp/destination'
STORED AS ORC
SELECT * FROM test_table;

INSERT OVERWRITE LOCAL DIRECTORY '/tmp/destination'
ROW FORMAT DELIMITED FIELDS TERMINATED BY ','
SELECT * FROM test_table;
