"""Tests for the rules module.

Where possible, tests should be contained within the
yaml test cases. For python based tests where this is
not possible, the rule code should be in the test file
name.
"""
