CREATE TRIGGER foo BEFORE INSERT ON bar EXECUTE PROCEDURE proc(args);

CREATE TRIGGER foo BEFORE INSERT on bar EXECUTE PROCEDURE proc(args);

CREATE TRIGGER foo AFTER UPDATE OF bar, baz ON bar EXECUTE PROCEDURE proc(args);

CREATE TRIGGER foo INSTEAD OF DELETE ON bar FROM baz DEFERRABLE INITIALLY IMMEDIATE FOR EACH ROW EXECUTE PROCEDURE proc(args);

CREATE TRIGGER foo BEFORE INSERT ON bar WHEN (a=b) EXECUTE PROCEDURE proc(args);

