use role my_role;

use warehouse my_warehouse;

use database my_database;

use schema my_schema;

USE ROLE "MY_ROLE";

USE WAREHOUSE "MY_WAREHOUSE";

USE DATABASE "MY_DATABASE";

USE "MY_DATABASE";

USE SCHEMA "MY_DATABASE"."MY_SCHEMA";

USE SCHEMA "MY_SCHEMA";

USE "MY_DATABASE"."MY_SCHEMA";

USE SECONDARY ROLES ALL;

USE SECONDARY ROLES NONE;
