DROP MATERIALIZED VIEW bar;
DROP MATERIALIZED VIEW foo, bar;
DROP MATERIALIZED VIEW bar CASCADE;
DROP MATERIALIZED VIEW foo, bar CASCADE;
DROP MATERIALIZED VIEW bar RESTRICT;
DROP MATERIALIZED VIEW foo, bar RESTRICT;
DROP MATERIALIZED VIEW IF EXISTS bar;
DROP MATERIALIZED VIEW IF EXISTS foo, bar;
DROP MATERIALIZED VIEW IF EXISTS bar CASCADE;
DROP MATERIALIZED VIEW IF EXISTS foo, bar CASCADE;
DROP MATERIALIZED VIEW IF EXISTS bar RESTRICT;
DROP MATERIALIZED VIEW IF EXISTS foo, bar RESTRICT;
