INSERT INTO books (title, author)
VALUES ('The Great Gatsby', 'F. Scott Fitzgerald');

INSERT books (title, author)
VALUES ('The Great Gatsby', 'F. Scott Fitzgerald');

INSERT INTO `project.dataset.table` (
    SELECT *
    FROM table1
);
