CREATE COLLATION numeric (provider = icu, locale = 'en@colNumeric=yes');

CREATE COLLATION french (locale = 'fr_FR.utf8');

CREATE COLLATION german_phonebook (provider = icu, locale = 'de-u-co-phonebk');

CREATE COLLATION german FROM "de_DE";
