SET LOCAL search_path = DEFAULT;
SET search_path TO my_schema, public;
SET datestyle TO postgres, dmy;
SET SESSION datestyle TO postgres, 'dmy';
SET TIME ZONE 'PST8PDT';
SET TIME ZONE 'Europe/Rome';
SET TIME ZONE LOCAL;
SET TIME ZONE DEFAULT;
