DROP POLICY account_managers ON accounts;
DROP POLICY IF EXISTS account_managers ON accounts;
DROP POLICY account_managers ON accounts CASCADE;
DROP POLICY account_managers ON accounts RESTRICT;
DROP POLICY IF EXISTS account_managers ON accounts RESTRICT;
