create group admin_group;

create group "admin_group";

create group admin_group user admin1;
create group admin_group with user admin1;

create group admin_group user admin1, admin2;
create group admin_group with user admin1, admin2;
