CREATE SEQUENCE foo;

CREATE SEQUENCE foo INCREMENT BY 3;

CREATE SEQUENCE foo MINVALUE 5 NO MAXVALUE;

CREATE SEQUENCE foo NO MINVALUE MAXVALUE 12;

CREATE SEQUENCE foo INCREMENT BY 5 START WITH 8 CACHE 4;

CREATE SEQUENCE foo NOCACHE;

CREATE SEQUENCE foo NOCYCLE ORDER;

CREATE SEQUENCE foo NOORDER;
