SELECT *
FROM source TABLESAMPLE(BUCKET 3 OUT OF 32 ON id) s;

SELECT *
FROM source TABLESAMPLE(BUCKET 3 OUT OF 32 ON rand()) s;

SELECT *
FROM source TABLESAMPLE(0.1 PERCENT) AS s;

SELECT *
FROM source TABLESAMPLE(100M) s;

SELECT * FROM source TABLESAMPLE(10 ROWS);
