SELECT
    t1.field,
    EXTRACT(year FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(y FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yyy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yyyy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yr FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(years FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yrs FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(month FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(mm FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(mon FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(mons FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(months FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(day FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(d FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dd FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(days FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dayofmonth FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dayofweek FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(weekday FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dow FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dw FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dayofweekiso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(weekday_iso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dow_iso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dw_iso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dayofyear FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yearday FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(doy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(dy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(week FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(w FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(wk FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(weekofyear FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(woy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(wy FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(weekiso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(week_iso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(weekofyeariso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(weekofyear_iso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(quarter FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(q FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(qtr FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(qtrs FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(quarters FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yearofweek FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(yearofweekiso FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(hour FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(h FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(hh FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(hr FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(hours FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(hrs FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(minute FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(m FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(mi FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(min FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(minutes FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(mins FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(second FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(s FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(sec FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(seconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(secs FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(millisecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(ms FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(msec FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(milliseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(microsecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(us FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(usec FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(microseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nanosecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(ns FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nsec FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nanosec FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nsecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nanoseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nanosecs FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(nseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_second FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_seconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_millisecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_milliseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_microsecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_microseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_nanosecond FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(epoch_nanoseconds FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(timezone_hour FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(tzh FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(timezone_minute FROM t1.sometime) AS a
FROM t1;

SELECT
    t1.field,
    EXTRACT(tzm FROM t1.sometime) AS a
FROM t1;
