vacuum;

vacuum sales;

vacuum sales to 100 percent;

vacuum recluster sales;

vacuum sort only sales to 75 percent;

vacuum delete only sales to 75 percent;

vacuum reindex listing;

vacuum reindex listing to 75 percent;

vacuum listing to 75 percent BOOST;
