DROP CONNECTION IF EXISTS name CASCADE;
DROP CLUSTER IF EXISTS name CASCADE;
DROP CLUSTER REPLICA IF EXISTS name CASCADE;
DROP DATABASE IF EXISTS name CASCADE;
DROP INDEX IF EXISTS name CASCADE;
DROP MATERIALIZED VIEW IF EXISTS name CASCADE;
DROP ROLE IF EXISTS name CASCADE;
DROP SECRET IF EXISTS name CASCADE;
DROP SCHEMA IF EXISTS name CASCADE;
DROP SINK IF EXISTS name CASCADE;
DROP SOURCE IF EXISTS name CASCADE;
DROP TABLE IF EXISTS name CASCADE;
DROP TYPE IF EXISTS name CASCADE;
DROP VIEW IF EXISTS name CASCADE;
DROP USER IF EXISTS name CASCADE;

DROP CONNECTION name;
DROP CLUSTER name;
DROP CLUSTER REPLICA name;
DROP DATABASE name;
DROP INDEX name;
DROP MATERIALIZED VIEW name;
DROP ROLE name;
DROP SECRET name;
DROP SCHEMA name;
DROP SINK name;
DROP SOURCE name;
DROP TABLE name;
DROP TYPE name;
DROP VIEW name;
DROP USER name;
