CREATE SEQUENCE SEQ_MELDER
START WITH 1
INCREMENT BY 1
GO

CREATE SEQUENCE Test.DecSeq
    AS decimal(3,0)
    START WITH 125
    INCREMENT BY 25
    MINVALUE 100
    MAXVALUE 200
    CYCLE
    CACHE 3
;
