DROP PROCEDURE do_db_maintenance();

drop procedure insert_actor;

drop procedure insert_actor(varchar);

drop procedure insert_actor(varchar, varchar);

drop procedure delete_actor, update_actor;

drop procedure delete_actor, update_actor CASCADE;

drop procedure delete_actor(in id varchar);

drop procedure insert_actor(varchar, varchar), insert_actor2(varchar, varchar);
