# Classic Inline Comment
SELECT 1; -- Classic Inline Comment
SELECT 1; # Classic Inline Comment
SELECT 1; //Snowflake Inline Comment
SELECT 1;-- Classic Inline Comment No Space
SELECT 1;# Classic Inline Comment No Space
SELECT 1//Snowflake Inline Comment No Space
