OPTIMIZE TABLE some_table;

OPTIMIZE TABLE some_table1, some_table2;

OPTIMIZE NO_WRITE_TO_BINLOG TABLE some_table;

OPTIMIZE NO_WRITE_TO_BINLOG TABLE some_table1, some_table2;

OPTIMIZE LOCAL TABLE some_table;

OPTIMIZE LOCAL TABLE some_table1, some_table2;
