SELECT
    col1,
    EXTRACT(MICROSECOND FROM col1),
    EXTRACT(MILLISECOND FROM col1),
    EXTRACT(SECOND FROM col1),
    EXTRACT(MINUTE FROM col1),
    EXTRACT(HOUR FROM col1),
    EXTRACT(DAY FROM col1),
    EXTRACT(DAYOFWEEK FROM col1),
    EXTRACT(DAYOFYEAR FROM col1),
    EXTRACT(WEEK FROM col1),
    EXTRACT(ISOWEEK FROM col1),
    EXTRACT(MONTH FROM col1),
    EXTRACT(QUARTER FROM col1),
    EXTRACT(YEAR FROM col1),
    EXTRACT(ISOYEAR FROM col1),
    EXTRACT(WEEK(SUNDAY) FROM col1),
    EXTRACT(WEEK FROM col1) AS week,
    EXTRACT(DATE FROM col1) AS week,
    EXTRACT(DATE FROM TIMESTAMP_SECONDS(1651135778)),
    LAST_DAY(col1, MONTH),
    LAST_DAY(col1, WEEK),
    LAST_DAY(col1, WEEK(SUNDAY)),
FROM tbl1;
