\echo "thing"

\echo "thing"

\x \\

\echo "thing"

SELECT 1; SELECT 2;

SELECT 1 + 3;

SELECT 1;

\echo "thing" \\ SELECT 1;

\echo "thing" \echo "thing2"

\prompt 'Region (1 - quebec, 2 - east, 3 - west): ' region_number
