-- List the columns of `customer` table in current database.
SHOW COLUMNS IN customer;

-- List the columns of `customer` table in `salesdb` database.
SHOW COLUMNS IN salesdb.customer;

-- List the columns of `customer` table in `salesdb` database
SHOW COLUMNS IN customer IN salesdb;
