select date_part(millennium, foo) from tbl1;

select date_part(millennia, foo) from tbl1;

select date_part(mil, foo) from tbl1;

select date_part(mils, foo) from tbl1;

select date_part(century, foo) from tbl1;

select date_part(centuries, foo) from tbl1;

select date_part(c, foo) from tbl1;

select date_part(cent, foo) from tbl1;

select date_part(cents, foo) from tbl1;

select date_part(decade, foo) from tbl1;

select date_part(decades, foo) from tbl1;

select date_part(dec, foo) from tbl1;

select date_part(decs, foo) from tbl1;

select extract(epoch from foo) from tbl1;

select extract(year from foo) from tbl1;

select extract(years from foo) from tbl1;

select extract(y from foo) from tbl1;

select extract(yr from foo) from tbl1;

select extract(yrs from foo) from tbl1;

select extract(quarter from foo) from tbl1;

select extract(quarters from foo) from tbl1;

select extract(qtr from foo) from tbl1;

select extract(qtrs from foo) from tbl1;

select extract(month from foo) from tbl1;

select extract(months from foo) from tbl1;

select extract(mon from foo) from tbl1;

select extract(mons from foo) from tbl1;

select extract(week from foo) from tbl1;

select extract(weeks from foo) from tbl1;

select extract(w from foo) from tbl1;

select extract(dayofweek from foo) from tbl1;

select extract(dow from foo) from tbl1;

select extract(dw from foo) from tbl1;

select extract(weekday from foo) from tbl1;

select extract(dayofyear from foo) from tbl1;

select extract(doy from foo) from tbl1;

select extract(dy from foo) from tbl1;

select extract(yearday from foo) from tbl1;

select extract(day from foo) from tbl1;

select extract(days from foo) from tbl1;

select extract(d from foo) from tbl1;

select extract(hour from foo) from tbl1;

select extract(hours from foo) from tbl1;

select extract(h from foo) from tbl1;

select extract(hr from foo) from tbl1;

select extract(hrs from foo) from tbl1;

select extract(minute from foo) from tbl1;

select extract(minutes from foo) from tbl1;

select extract(m from foo) from tbl1;

select extract(min from foo) from tbl1;

select extract(mins from foo) from tbl1;

select extract(second from foo) from tbl1;

select extract(seconds from foo) from tbl1;

select dateadd(s, 123, foo) from tbl1;

select dateadd(sec, 123, foo) from tbl1;

select dateadd(secs, 123, foo) from tbl1;

select dateadd(millisecond, 123, foo) from tbl1;

select dateadd(milliseconds, 123, foo) from tbl1;

select dateadd(ms, 123, foo) from tbl1;

select dateadd(msec, 123, foo) from tbl1;

select dateadd(msecs, 123, foo) from tbl1;

select dateadd(msecond, 123, foo) from tbl1;

select dateadd(mseconds, 123, foo) from tbl1;

select dateadd(millisec, 123, foo) from tbl1;

select dateadd(millisecs, 123, foo) from tbl1;

select dateadd(millisecon, 123, foo) from tbl1;

select dateadd(microsecond, 123, foo) from tbl1;

select dateadd(microseconds, 123, foo) from tbl1;

select datediff(microsec, foo, bar) from tbl1;

select datediff(microsecs, foo, bar) from tbl1;

select datediff(microsecond, foo, bar) from tbl1;

select datediff(usecond, foo, bar) from tbl1;

select datediff(useconds, foo, bar) from tbl1;

select datediff(us, foo, bar) from tbl1;

select datediff(usec, foo, bar) from tbl1;

select datediff(usecs, foo, bar) from tbl1;

select datediff(timezone, foo, bar) from tbl1;

select datediff(timezone_hour, foo, bar) from tbl1;

select datediff(timezone_minute, foo, bar) from tbl1;
