select * from u as t ( a, b );
select * from u as t (a, b);
select * from u as t(a,b);
select * from u as (a,b);
select * from u t ( a, b );
select * from u t (a, b);
select * from u t(a,b);
select * from u (a,b);
