comment on column my_table.my_column is 'comment';
comment on table foo is 'comment';
comment on view foo is 'comment';
comment on schema foo is 'comment';
comment on database foo is 'comment';
comment on warehouse foo is 'comment';
comment on user foo is 'comment';
comment on stage foo is 'comment';
comment on function foo is 'comment';
comment on procedure foo is 'comment';
comment on sequence foo is 'comment';
comment on share foo is 'comment';
comment on pipe foo is 'comment';
comment on stream foo is 'comment';
comment on task foo is 'comment';
comment on network policy foo is 'comment';
comment on api integration foo is 'comment';
comment on notification integration foo is 'comment';
comment on security integration foo is 'comment';
comment on storage integration foo is 'comment';
comment on session policy foo is 'comment';
comment on external table foo is 'comment';
comment on materialized view foo is 'comment';
comment on masking policy foo is 'comment';
comment on row access policy foo is 'comment';
comment on file format foo is 'comment';
