ALTER SHARE MY_SHARE ADD ACCOUNTS = my_account_1;
ALTER SHARE IF EXISTS MY_SHARE ADD ACCOUNTS = my_account_1;
ALTER SHARE MY_SHARE REMOVE ACCOUNTS = my_account_1;
ALTER SHARE MY_SHARE ADD ACCOUNTS = my_account_1, my_account_2;
ALTER SHARE MY_SHARE ADD ACCOUNTS = my_account_1, my_account_2, my_account_3;
ALTER SHARE MY_SHARE SET TAG tag1 = 'value1';
ALTER SHARE IF EXISTS MY_SHARE SET TAG tag1 = 'value1', tag2 = 'value2';
ALTER SHARE MY_SHARE UNSET TAG tag1;
ALTER SHARE MY_SHARE UNSET TAG tag1, tag2;
ALTER SHARE MY_SHARE UNSET COMMENT;
ALTER SHARE MY_SHARE ADD ACCOUNTS = my_account_1 SHARE_RESTRICTIONS = TRUE;
ALTER SHARE MY_SHARE ADD ACCOUNTS = my_account_1, my_account_2 SHARE_RESTRICTIONS = FALSE;
ALTER SHARE MY_SHARE SET ACCOUNTS = my_account_1 COMMENT = 'my_comment';
ALTER SHARE IF EXISTS MY_SHARE SET ACCOUNTS = my_account_1, my_account_2 COMMENT = 'my_comment';
