REPAIR TABLE some_table;

REPAIR TABLE some_table1, some_table2;

REPAIR NO_WRITE_TO_BINLOG TABLE some_table;

REPAIR NO_WRITE_TO_BINLOG TABLE some_table1, some_table2;

REPAIR LOCAL TABLE some_table;

REPAIR LOCAL TABLE some_table1, some_table2;

REPAIR TABLE some_table QUICK;

REPAIR TABLE some_table EXTENDED;

REPAIR TABLE some_table USE_FRM;

REPAIR TABLE some_table QUICK EXTENDED USE_FRM;
