select `delimited``with escaped` from `some``table` `some``alias`;
select `delimited``with escaped` from `some``table` as `some``alias`;
select ```delimited` `alias` FROM `some``table` `some````alias`;
select `delimited``` `alias` FROM `some``table` ```some````alias```;

SELECT `delimited ``identifier` `alias` FROM `some``table` `some``alias`;
SELECT `delimited ``identifier` AS `alias` FROM `some``table` AS `some``alias`;
