
SHOW COLUMNS;
SHOW CONNECTIONS;
SHOW CLUSTERS;
SHOW CLUSTER REPLICAS;
SHOW DATABASES;
SHOW INDEXES;
SHOW MATERIALIZED VIEWS;
SHOW SECRETS;
SHOW SCHEMAS;
SHOW SINKS;
SHOW SOURCES;
SHOW TABLES;
SHOW TYPES;
SHOW VIEWS;
SHOW OBJECTS;

SHOW CREATE CONNECTION connection_name;
SHOW CREATE INDEX index_name;
SHOW CREATE MATERIALIZED VIEW view_name;
SHOW CREATE SINK sink_name;
SHOW CREATE SOURCE source_name;
SHOW CREATE TABLE table_name;
SHOW CREATE VIEW view_name;

SHOW TABLES LIKE 'v%';
SHOW DATABASES WHERE database_name = 'database_name';
SHOW SECRETS WHERE name='secret_name';
SHOW SECRETS LIKE 'secret_name%';
SHOW COLUMNS FROM some_table WHERE name='column_name';
SHOW SECRETS FROM some_schema WHERE name='secret_name';
SHOW SECRETS FROM some_schema WHERE LIKE 'name%';
SHOW SCHEMAS FROM database_name;
SHOW SINKS FROM some_schema;
SHOW SOURCES FROM some_schema;
SHOW TABLES FROM some_schema;
SHOW TYPES FROM some_schema;
SHOW VIEWS FROM some_schema;
SHOW OBJECTS FROM some_schema;

SHOW INDEXES IN CLUSTER bar WHERE name NOT LIKE 'mz_%';
SHOW MATERIALIZED VIEWS IN CLUSTER other;
