COMMENT ON SCHEMA s1 IS 'My first schema';
COMMENT ON TABLE a_schema.t1 IS 'My first table';
COMMENT ON t1 (id IS 'Identity column', zip IS 'Zip code');
COMMENT ON SCRIPT script1 IS 'My first script';
COMMENT ON CONSUMER GROUP admin_group IS 'VERY important!!!';
