ALTER CONSUMER GROUP TEST_TEAM SET
 PRECEDENCE = '800',
 CPU_WEIGHT = '150',
 GROUP_TEMP_DB_RAM_LIMIT = '10G',
 SESSION_TEMP_DB_RAM_LIMIT = '5G',
 QUERY_TIMEOUT = 60,
 IDLE_TIMEOUT = 3600;

ALTER CONSUMER GROUP "ADMIN" SET
 PRECEDENCE = '800';
