-- Testing of the specific create table begin options
CREATE MULTISET TABLE CONSUMOS, NO FALLBACK, NO BEFORE JOURNAL, NO AFTER JOURNAL,
     CHECKSUM = DEFAULT, DEFAULT MERGEBLOCKRATIO
(
    FIELD1 CHAR(9)
)
PRIMARY INDEX( FIELD1 );
