-- EXCEPT
SELECT c FROM number1 EXCEPT SELECT c FROM number2;

-- EXCEPT ALL
SELECT c FROM number1 EXCEPT ALL (SELECT c FROM number2);

-- MINUS
SELECT c FROM number1 MINUS SELECT c FROM number2;

-- MINUS ALL
SELECT c FROM number1 MINUS ALL (SELECT c FROM number2);

-- INTERSECT
(SELECT c FROM number1) INTERSECT (SELECT c FROM number2);

-- INTERSECT DISTINCT
(SELECT c FROM number1) INTERSECT DISTINCT (SELECT c FROM number2);

-- INTERSECT ALL
(SELECT c FROM number1) INTERSECT ALL (SELECT c FROM number2);

-- UNION
(SELECT c FROM number1) UNION (SELECT c FROM number2);

-- UNION DISTINCT
(SELECT c FROM number1) UNION DISTINCT (SELECT c FROM number2);

-- UNION ALL
SELECT c FROM number1 UNION ALL (SELECT c FROM number2);
