DROP FUNCTION sqrt(integer);
DROP FUNCTION sqrt(integer), sqrt(bigint);
DROP FUNCTION update_employee_salaries;
DROP FUNCTION update_employee_salaries();
DROP FUNCTION IF EXISTS foo (IN my_var integer, VARIADIC my_var_2 text);
